@@ -168,11 +168,11 @@ int main(int argc, char*argv[]) {
                    exit (1);
 
                 }
-		if (!userid[*uid]) 
-                {     
+		if (!userid[*uid])
+                {
 		    lseek(fh_lastlog, (long)*uid * sizeof (struct lastlog), 0);
-		    if ((wtmp_bytes_read = read(fh_lastlog, &lastlog_ent, sizeof (struct lastlog))) > 0) 
-                    { 
+		    if ((wtmp_bytes_read = read(fh_lastlog, &lastlog_ent, sizeof (struct lastlog))) > 0)
+                    {
                         if (wtmp_bytes_read < sizeof(struct lastlog))
                         {
                            fprintf(stderr, "lastlog entry may be corrupted");
@@ -191,9 +191,11 @@ int main(int argc, char*argv[]) {
                         userid[*uid]=TRUE;
                     }
 		}
-           }      
+           }
 	}
+#if 0
 	printf("\n");
+#endif
 	free_results(localpwd);
 	close(fh_wtmp);
 	close(fh_lastlog);
@@ -289,5 +291,3 @@ int getslot(struct s_localpwd *localpwd, uid_t uid)
         }
         return -1;
 }
-
-
