@@ -1,6 +1,6 @@
 /*
-  (C) Nelson Murilo - 2003/01/31
-  Version 0.7
+  (C) Nelson Murilo - 2003/06/07
+  Version 0.8
   C port from chkproc.pl code from Klaus Steding-Jessen <jessen@nic.br>
   and Cristine Hoepers <cristine@nic.br> +little output changes.
 
@@ -20,6 +20,16 @@
   - ported to SunOS.
   
   2003/01/19 - Another Adore based lkm test. Original idea from Junichi Murakami
+
+  2003/02/02 - More little fixes - Nelson Murilo
+
+  2003/02/23 - Use of kill to eliminate false-positives abandonated, It is
+  preferable false-positives that false-negatives. Uncomment kill() functions 
+  if you like  it. 
+
+  2003/06/07 - Fix for NPTL threading mechanisms - patch by Mike Griego 
+
+  2003/09/01 - Fix for ps mode detect, patch by Bill Dupree and others 
  
 */
 
@@ -47,12 +57,15 @@ main (){ return 0; }
 #if defined(__sun)
 #define PS "ps -edf"
 #else
-#define PS "ps mauxwww 2> /dev/null"
-#define PS1 "ps auxwww"
+#define PS  "ps mauxwww 2>/dev/null"
 #endif
+#define PS1 "ps auxwww"
 
 int psproc [MAX_PROCESSES+1];
 int dirproc[MAX_PROCESSES+1];
+#if defined(__linux__)
+int isathread[MAX_PROCESSES+1];
+#endif
 
 main(int argc, char **argv)
 {
@@ -63,6 +76,10 @@ main(int argc, char **argv)
    int i, j, retps, retdir;
    int verbose = 0;
    long ret = 0;
+   char * tmp_d_name;
+#if defined(__linux__)
+   int maybeathread;
+#endif
 #if defined(__sun)
    psinfo_t psbuf;
 #endif
@@ -83,7 +100,7 @@ main(int argc, char **argv)
 	return 0;
       }
    }
-
+   *buf = 0;
    fgets(buf, MAX_BUF, ps); /* Skip header */
 #if defined(__sun)
    if (!isspace(*buf))
@@ -91,7 +108,13 @@ main(int argc, char **argv)
    if (!isalpha(*buf))
 #endif
    {
+     fclose(ps); 
      ps  = popen(PS1, "r");
+     if (!ps) 
+     {
+        perror("ps"); 
+        exit(1);
+     }
      fgets(buf, MAX_BUF, ps); /* Skip header */
      if (!isalpha(*buf))
      {
@@ -99,8 +122,12 @@ main(int argc, char **argv)
         exit(2);
      }
    }
-   for (i = FIRST_PROCESS; i <= MAX_PROCESSES; i++) /* Init matrix */
+   for (i = FIRST_PROCESS; i <= MAX_PROCESSES; i++) { /* Init matrix */
      psproc[i] = dirproc[i] = 0;
+#if defined(__linux__)
+     isathread[i] = 0;
+#endif
+   }
 
    while (fgets(buf, MAX_BUF, ps))
    {
@@ -116,7 +143,7 @@ main(int argc, char **argv)
       ret = atol(p);
       if ( ret < 0 || ret > MAX_PROCESSES )
       {
-         fprintf (stderr, " OooPS, not expected %d value\n");
+         fprintf (stderr, " OooPS, not expected %d value\n", ret);
          exit (2);
       }
       psproc[ret] = 1;
@@ -125,10 +152,29 @@ main(int argc, char **argv)
 
    while (dir = readdir(proc))
    {
-      if (!strcmp(dir->d_name, ".") || !strcmp(dir->d_name, "..") ||
-         !isdigit(*dir->d_name))
+#if defined(__linux__)
+      maybeathread = 0;
+#endif
+      tmp_d_name = dir->d_name;
+      if (!strcmp(tmp_d_name, ".") || !strcmp(tmp_d_name, ".."))
          continue;
-      dirproc[atol(dir->d_name)] = 1;
+#if defined(__linux__)
+      if (*tmp_d_name == '.') { /* here we catch the new NTPL threads in linux.  They are listed in /proc as PIDs with a period prepended */
+         tmp_d_name++;
+         maybeathread = 1;
+      }
+#endif
+      if(!isdigit(*tmp_d_name))
+         continue;
+#if defined(__linux__)
+      else if (maybeathread) {
+         isathread[atol(tmp_d_name)] = 1; /* mark it as a linux NTPL thread if it's in the form of "\.[0-9]*" */
+         if (verbose)
+            printf("%d is a Linux Thread, marking as such...\n", atol(tmp_d_name));
+      }
+#endif
+
+      dirproc[atol(tmp_d_name)] = 1;
    }
    closedir(proc);
 
@@ -140,18 +186,33 @@ main(int argc, char **argv)
       snprintf(&buf[6], 6, "%d", i);
       if (!chdir(buf))
       {
-         if (!dirproc[i] && !kill(i, 0))
+         if (!dirproc[i] ) // && !kill(i, 0))
          {
+#if defined(__linux__)
+            if (!isathread[i]) {
+#endif
             retdir++;
             if (verbose)
 	       printf ("PID %5d: not in readdir output\n", i);
+#if defined(__linux__)
+            }
+#endif
          }
-         if (!psproc[i] && !kill(i, 0))
+         if (!psproc[i] ) // && !kill(i, 0))
          {
+#if defined(__linux__)
+            if(!isathread[i]) {
+#endif
             retps++;
             if (verbose)
 	       printf ("PID %5d: not in ps output\n", i);
+#if defined(__linux__)
+            }
+#endif
 	 }
+#if defined(__linux__)
+         if(!isathread[i]) {
+#endif
 	 if ((!dirproc[i] || !psproc[i]) && !kill(i, 0) && (verbose > 1))
 	 {
 #if defined(__linux__)
@@ -178,6 +239,9 @@ main(int argc, char **argv)
                printf ("PSINFO %5d: unknown\n", i);
 #endif
          }
+#if defined(__linux__)
+         }
+#endif
       }
    }
    if (retdir)
