@@ -1,19 +1,21 @@
 /*
    ifpromisc - This is a simple subset of Fred N. van Kempen, 
-               <waltje@uwalt.nl.mugnet.org>'s ifconfig.
+               <waltje@uwalt.nl.mugnet.org>'s ifconfig and iplink code. 
                Show state of all ethernet interface 
                xxx is PROMISC 
                or
                xxx is not promisc
 
-   Version:	 @(#)ifpromisc.c	0.6	2002/07/27
-   Last Changes: Ports for Solaris  
+   Version:	 @(#)ifpromisc.c	0.7	2003/06/07
+   Last Changes: Fix for newer linux kernels
+                 Minor fixes
+                 Ports for Solaris  
                  Andre Gustavo <gustavo@anita.visualnet.com.br> 
                  Port for OpenBSD 
          	 Nelson Murilo, <nelson@pangeia.com.br>
   
    Author:	Nelson Murilo, <nelson@pangeia.com.br>
-  		Copyright 1997-2002 (C) Pangeia Informatica
+  		Copyright 1997-2003 (C) Pangeia Informatica
   
   		This program is free software; you can redistribute it
   		and/or  modify it under  the terms of  the GNU General
@@ -52,31 +54,22 @@ struct interface
   short			flags;			/* various flags	*/
 };
 
-/* This structure defines protocol families and their handlers. */
-struct aftype {
-  char		*name;
-  char		*title;
-  int		af;
-  int		alen;
-  char		*(*print)	(unsigned char *);
-  char		*(*sprint)	(struct sockaddr *, int numeric);
-  int		(*input)	(char *bufp, struct sockaddr *);
-  void		(*herror)	(char *text);
-};
-  
 char *Release = "chkrootkit package",
-     *Version = "@(#) ifpromisc 0.5 (2002/07/27)";
+     *Version = "@(#) ifpromisc 0.7 (2003/06/07)";
 
-int skfd = -1;				/* AF_INET raw socket desc.	*/
+int skfd = -1;				/* AF_INET or AF_PACKET raw socket desc.	*/
+int q = 0;                              /* Quiet mode on or off				*/
 
 static void ife_print(struct interface *ptr)
 {
-  printf("%s is ", ptr->name);
-  if (ptr->flags & IFF_PROMISC) 
-     printf("PROMISC"); 
-  else
-     printf("not promisc"); 
-  putchar('\n');
+   if (ptr->flags & IFF_PROMISC) 
+      printf("%s is %s", ptr->name, "PROMISC");
+   else
+   {
+      if (!q)
+         printf("%s is %s", ptr->name, "not promisc"); 
+   }    
+   putchar('\n');
 }
 
 /* Fetch the inteface configuration from the kernel. */
@@ -88,18 +81,19 @@ static int if_fetch(char *ifname, struct interface *ife)
   strcpy(ife->name, ifname);
 
   strcpy(ifr.ifr_name, ifname);
-  if (ioctl(skfd, SIOCGIFFLAGS, &ifr) < 0) return(-1);
+  if (ioctl(skfd, SIOCGIFFLAGS, &ifr) < 0) 
+      return(-1);
   ife->flags = ifr.ifr_flags;
   return(0);
 }
 
 static void if_print()
 {
-  char buff[1024];
-  struct interface ife;
-  struct ifconf ifc;
-  struct ifreq *ifr;
-  int i;
+   char buff[1024];
+   struct interface ife;
+   struct ifconf ifc;
+   struct ifreq *ifr;
+   int i;
 
    ifc.ifc_len = sizeof(buff);
    ifc.ifc_buf = buff;
@@ -127,11 +121,11 @@ static void if_print()
 
 int main(int argc, char **argv)
 {
-  struct sockaddr sa;
-  struct aftype *ap;
+  if (argc == 2 && !memcmp(argv[1], "-q", 2))
+     q++;
 
   /* Create a channel to the NET kernel. */
-  if ((skfd = socket(AF_INET, SOCK_DGRAM,0)) < 0) {
+   if ((skfd = socket(AF_INET, SOCK_DGRAM,0)) < 0) {
 	perror("socket");
 	exit(-1);
   }
